@@ -1358,6 +1358,10 @@ static void ar5523_configure_filter(struct ieee80211_hw *hw,
 }
 
 static const struct ieee80211_ops ar5523_ops = {
+	.add_chanctx = ieee80211_emulate_add_chanctx,
+	.remove_chanctx = ieee80211_emulate_remove_chanctx,
+	.change_chanctx = ieee80211_emulate_change_chanctx,
+	.switch_vif_chanctx = ieee80211_emulate_switch_vif_chanctx,
 	.start			= ar5523_start,
 	.stop			= ar5523_stop,
 	.tx			= ar5523_tx,
@@ -1590,20 +1594,6 @@ static int ar5523_probe(struct usb_interface *intf,
 	struct ar5523 *ar;
 	int error = -ENOMEM;
 
-	static const u8 bulk_ep_addr[] = {
-		AR5523_CMD_TX_PIPE | USB_DIR_OUT,
-		AR5523_DATA_TX_PIPE | USB_DIR_OUT,
-		AR5523_CMD_RX_PIPE | USB_DIR_IN,
-		AR5523_DATA_RX_PIPE | USB_DIR_IN,
-		0};
-
-	if (!usb_check_bulk_endpoints(intf, bulk_ep_addr)) {
-		dev_err(&dev->dev,
-			"Could not find all expected endpoints\n");
-		error = -ENODEV;
-		goto out;
-	}
-
 	/*
 	 * Load firmware if the device requires it.  This will return
 	 * -ENXIO on success and we'll get called back afer the usb
